// smart_assert_settings.hpp

// Boost.SmartAssert library
//
// Copyright (C) 2003 John Torjo (john@torjo.com)
//
// Permission to copy, use, sell and distribute this software is granted
// provided this copyright notice appears in all copies.
// Permission to modify the code and to distribute modified code is granted
// provided this copyright notice appears in all copies, and a notice
// that the code was modified is included with the copyright notice.
//
// This software is provided "as is" without express or implied warranty,
// and with no claim as to its suitability for any purpose.
 
// See http://www.boost.org for updates, documentation, and revision history.

#ifndef BOOST_INNER_SMART_ASSERT_SETTINGS_HPP_INCLUDED
#define BOOST_INNER_SMART_ASSERT_SETTINGS_HPP_INCLUDED

#include <map>
#include <iostream>
#include <boost/function.hpp>
#include "smart_assert_context.hpp"
#include "smart_assert_handlers.hpp"

namespace boost {

namespace smart_assert {

// forward declaration(s)
class smart_assert_settings;
smart_assert_settings & assert_settings();

/*
    allows manipulating the assert settings

    // FIXME - make all public operations thread-safe !

*/
class smart_assert_settings {

    friend smart_assert_settings & assert_settings();

    // level handler function
    typedef boost::function1< void, const boost::smart_assert::assert_context&> level_handler_func;

    // use boost::smart_assert::assert_settings() to manipulate it
    smart_assert_settings() {
        // FIXME - in "release" mode, we shold have different handlers!!! - TOTHINK
        set_level_handler( lvl_debug, &default_handle_debug);
        set_level_handler( lvl_warn, &default_handle_warn);
        set_level_handler( lvl_error, &default_handle_error);
        set_level_handler( lvl_critical, &default_handle_critical);

        m_dumper = &std::cout;
    }
public:

    // sets the handler for a given ASSERTion level
    void set_level_handler( int nLevel, level_handler_func f) {
        m_level_handlers[ nLevel] = f;
    }


    // gets the handler for a given ASSERTion level
    //
    // recommandation: if used in client code, use
    // has_level_handler first!
    level_handler_func get_level_handler( int nLevel) {
        level_handlers_collection::const_iterator found = m_level_handlers.find( nLevel);
        if ( found != m_level_handlers.end() )
            return found->second;
        else {
            get_dumper() << "[SMART_ASSERT logical error] "
                << "handler not set for level " << nLevel 
                << ", setting it to same as debug's" << std::endl;
            found = m_level_handlers.find( lvl_debug);
            m_level_handlers[ nLevel] = found->second;
            return found->second;
        }
    }

    // returns true if we have a handler for a given ASSERTion level
    bool has_level_handler( int nLevel) const {
        level_handlers_collection::const_iterator found = m_level_handlers.find( nLevel);
        return found != m_level_handlers.end();
    }

    // in case we need to dump logical errors related 
    // to misuse of SMART_ASSERTs somewhere...
    //
    // note that throwing a std::logic_error would be pretty drastic,
    // for misuse of a SMART_ASSERT...
    void set_dumper( std::ostream & out) {
        m_dumper = &out;
    }

    std::ostream & get_dumper() const {
        return *m_dumper;
    }
private:

    typedef std::map< int, level_handler_func> level_handlers_collection;
    level_handlers_collection m_level_handlers;

    // where to dump logical errors related to using SMART_ASSERT
    // 
    // we can't assert, can we? ;-)
    //
    // note that throwing a std::logic_error would be pretty drastic,
    // for misuse of a SMART_ASSERT...
    std::ostream * m_dumper;
};


// allows manipulating the assert settings
smart_assert_settings &assert_settings() { 
    // FIXME - make sure it's initialized before main!
    static smart_assert_settings inst;
    return inst;
}


} // namespace smart_assert

} // namespace boost

#endif

